from django.urls import path

from .views import dashboard, dashboard_index

urlpatterns = [
    path("", dashboard_index, name="django_sql_dashboard-index"),
    path("<slug>/", dashboard, name="django_sql_dashboard-dashboard"),
]



from html import escape

from django.contrib import admin
from django.utils.safestring import mark_safe

from .models import Dashboard, DashboardQuery


class DashboardQueryInline(admin.TabularInline):
    model = DashboardQuery
    extra = 1

    def has_change_permission(self, request, obj=None):
        if obj is None:
            return True
        return obj.user_can_edit(request.user)

    def get_readonly_fields(self, request, obj=None):
        if not request.user.has_perm("django_sql_dashboard.execute_sql"):
            return ("sql",)
        else:
            return tuple()


@admin.register(Dashboard)
class DashboardAdmin(admin.ModelAdmin):
    list_display = ("slug", "title", "owned_by", "view_policy", "view_dashboard")
    inlines = [
        DashboardQueryInline,
    ]
    raw_id_fields = ("owned_by",)
    fieldsets = (
        (
            None,
            {"fields": ("slug", "title", "description", "owned_by", "created_at")},
        ),
        (
            "Permissions",
            {"fields": ("view_policy", "edit_policy", "view_group", "edit_group")},
        ),
    )

    def view_dashboard(self, obj):
        return mark_safe(
            '<a href="{path}">{path}</a>'.format(path=escape(obj.get_absolute_url()))
        )

    def save_model(self, request, obj, form, change):
        if not obj.owned_by_id:
            obj.owned_by = request.user
        obj.save()

    def has_change_permission(self, request, obj=None):
        if obj is None:
            return True
        if request.user.is_superuser:
            return True
        return obj.user_can_edit(request.user)

    def get_readonly_fields(self, request, obj):
        readonly_fields = ["created_at"]
        if not request.user.is_superuser:
            readonly_fields.append("owned_by")
        return readonly_fields



import csv
import hashlib
import re
import time
from io import StringIO
from urllib.parse import urlencode

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.db import connections
from django.db.utils import ProgrammingError
from django.forms import CharField, ModelForm, Textarea
from django.http.response import (
    HttpResponseForbidden,
    HttpResponseRedirect,
    StreamingHttpResponse,
)
from django.shortcuts import get_object_or_404, render

from .models import Dashboard
from .utils import (
    check_for_base64_upgrade,
    displayable_rows,
    extract_named_parameters,
    sign_sql,
    unsign_sql,
)

# https://github.com/simonw/django-sql-dashboard/issues/58
MAX_REDIRECT_LENGTH = 1800


class SaveDashboardForm(ModelForm):
    slug = CharField(required=False, label="URL", help_text='For example "daily-stats"')

    class Meta:
        model = Dashboard
        fields = (
            "title",
            "slug",
            "description",
            "view_policy",
            "view_group",
            "edit_policy",
            "edit_group",
        )
        widgets = {
            "description": Textarea(
                attrs={
                    "placeholder": "Optional description, shown at the top of the dashboard page"
                }
            )
        }


@login_required
def dashboard_index(request):
    if not request.user.has_perm("django_sql_dashboard.execute_sql"):
        return HttpResponseForbidden("You do not have permission to execute SQL")
    sql_queries = []
    too_long_so_use_post = False
    save_form = SaveDashboardForm(prefix="_save")
    if request.method == "POST":
        # Is this an export?
        if any(
            k for k in request.POST.keys() if k.startswith("export_")
        ) and request.user.has_perm("django_sql_dashboard.execute_sql"):
            if not getattr(settings, "DASHBOARD_ENABLE_FULL_EXPORT", None):
                return HttpResponseForbidden("The export feature is not enabled")
            return export_sql_results(request)
        sqls = [sql for sql in request.POST.getlist("sql") if sql.strip()]

        saving = False
        # How about a save?
        if request.POST.get("_save-slug"):
            save_form = SaveDashboardForm(request.POST, prefix="_save")
            saving = True
            if save_form.is_valid():
                dashboard = save_form.save(commit=False)
                dashboard.owned_by = request.user
                dashboard.save()
                for sql in sqls:
                    dashboard.queries.create(sql=sql)
                return HttpResponseRedirect(dashboard.get_absolute_url())

        # Convert ?sql= into signed values and redirect as GET
        other_pairs = [
            (key, value)
            for key, value in request.POST.items()
            if key not in ("sql", "csrfmiddlewaretoken")
            and not key.startswith("_save-")
        ]
        signed_sqls = [sign_sql(sql) for sql in sqls if sql.strip()]
        params = {
            "sql": signed_sqls,
        }
        params.update(other_pairs)
        redirect_path = request.path + "?" + urlencode(params, doseq=True)
        # Is this short enough for us to redirect?
        too_long_so_use_post = len(redirect_path) > MAX_REDIRECT_LENGTH
        if not saving and not too_long_so_use_post:
            return HttpResponseRedirect(redirect_path)
        else:
            sql_queries = sqls
    unverified_sql_queries = []
    for signed_sql in request.GET.getlist("sql"):
        sql, signature_verified = unsign_sql(signed_sql)
        if signature_verified:
            sql_queries.append(sql)
        else:
            unverified_sql_queries.append(sql)
    if getattr(settings, "DASHBOARD_UPGRADE_OLD_BASE64_LINKS", None):
        redirect_querystring = check_for_base64_upgrade(sql_queries)
        if redirect_querystring:
            return HttpResponseRedirect(request.path + redirect_querystring)
    return _dashboard_index(
        request,
        sql_queries,
        unverified_sql_queries=unverified_sql_queries,
        too_long_so_use_post=too_long_so_use_post,
        extra_context={"save_form": save_form},
    )


def _dashboard_index(
    request,
    sql_queries,
    unverified_sql_queries=None,
    title=None,
    description=None,
    dashboard=None,
    too_long_so_use_post=False,
    template="django_sql_dashboard/dashboard.html",
    extra_context=None,
):
    query_results = []
    alias = getattr(settings, "DASHBOARD_DB_ALIAS", "dashboard")
    row_limit = getattr(settings, "DASHBOARD_ROW_LIMIT", None) or 100
    connection = connections[alias]
    with connection.cursor() as tables_cursor:
        tables_cursor.execute(
            """
            with visible_tables as (
              select table_name
              from information_schema.tables
              where table_schema = 'public'
              order by table_name
            )
            select
              information_schema.columns.table_name,
              string_agg(column_name, ', ' order by ordinal_position) as columns
            from
              information_schema.columns
            join
              visible_tables on
              information_schema.columns.table_name = visible_tables.table_name
            where
              information_schema.columns.table_schema = 'public'
            group by
              information_schema.columns.table_name
            order by
              information_schema.columns.table_name
        """
        )
        available_tables = [
            {"name": t[0], "columns": t[1]} for t in tables_cursor.fetchall()
        ]

    parameters = []
    sql_query_parameter_errors = []
    for sql in sql_queries:
        try:
            extracted = extract_named_parameters(sql)
            for p in extracted:
                if p not in parameters:
                    parameters.append(p)
            sql_query_parameter_errors.append(False)
        except ValueError as e:
            if "%" in sql:
                sql_query_parameter_errors.append(
                    r"Invalid query - try escaping single '%' as double '%%'"
                )
            else:
                sql_query_parameter_errors.append(str(e))
    parameter_values = {
        parameter: request.POST.get(parameter, request.GET.get(parameter, ""))
        for parameter in parameters
        if parameter != "sql"
    }
    extra_qs = "&{}".format(urlencode(parameter_values)) if parameter_values else ""
    results_index = -1
    if sql_queries:
        for sql, parameter_error in zip(sql_queries, sql_query_parameter_errors):
            results_index += 1
            sql = sql.strip().rstrip(";")
            base_error_result = {
                "index": str(results_index),
                "sql": sql,
                "textarea_rows": min(5, len(sql.split("\n"))),
                "rows": [],
                "row_lists": [],
                "description": [],
                "columns": [],
                "column_details": [],
                "truncated": False,
                "extra_qs": extra_qs,
                "error": None,
                "templates": ["django_sql_dashboard/widgets/error.html"],
            }
            if parameter_error:
                query_results.append(
                    dict(
                        base_error_result,
                        error=parameter_error,
                    )
                )
                continue
            if ";" in sql:
                query_results.append(
                    dict(base_error_result, error="';' not allowed in SQL queries")
                )
                continue
            with connection.cursor() as cursor:
                duration_ms = None
                try:
                    cursor.execute("BEGIN;")
                    start = time.perf_counter()
                    # Running a SELECT prevents future SET TRANSACTION READ WRITE:
                    cursor.execute("SELECT 1;")
                    cursor.fetchall()
                    cursor.execute(sql, parameter_values)
                    try:
                        rows = list(cursor.fetchmany(row_limit + 1))
                    except ProgrammingError as e:
                        rows = [{"statusmessage": str(cursor.statusmessage)}]
                    duration_ms = (time.perf_counter() - start) * 1000.0
                except Exception as e:
                    query_results.append(dict(base_error_result, error=str(e)))
                else:
                    templates = ["django_sql_dashboard/widgets/default.html"]
                    columns = [c.name for c in cursor.description]
                    template_name = ("-".join(sorted(columns))) + ".html"
                    if len(template_name) < 255:
                        templates.insert(
                            0,
                            "django_sql_dashboard/widgets/" + template_name,
                        )
                    display_rows = displayable_rows(rows[:row_limit])
                    column_details = [
                        {"name": column, "is_unambiguous": columns.count(column) == 1}
                        for column in columns
                    ]
                    query_results.append(
                        {
                            "index": str(results_index),
                            "sql": sql,
                            "textarea_rows": len(sql.split("\n")),
                            "rows": [dict(zip(columns, row)) for row in display_rows],
                            "row_lists": display_rows,
                            "description": cursor.description,
                            "columns": columns,
                            "column_details": column_details,
                            "truncated": len(rows) == row_limit + 1,
                            "extra_qs": extra_qs,
                            "duration_ms": duration_ms,
                            "templates": templates,
                        }
                    )
                finally:
                    cursor.execute("ROLLBACK;")
    # Page title, composed of truncated SQL queries
    html_title = "SQL Dashboard"
    if sql_queries:
        html_title = "SQL: " + " [,] ".join(sql_queries)

    if dashboard and dashboard.title:
        html_title = dashboard.title

    # Add named parameter values, if any exist
    provided_values = {
        key: value for key, value in parameter_values.items() if value.strip()
    }
    if provided_values:
        if len(provided_values) == 1:
            html_title += ": {}".format(list(provided_values.values())[0])
        else:
            html_title += ": {}".format(
                ", ".join(
                    "{}={}".format(key, value) for key, value in provided_values.items()
                )
            )

    user_can_execute_sql = request.user.has_perm("django_sql_dashboard.execute_sql")

    saved_dashboards = []
    if not dashboard:
        # Only show saved dashboards on index page
        saved_dashboards = [
            (dashboard, dashboard.user_can_edit(request.user))
            for dashboard in Dashboard.get_visible_to_user(request.user).select_related(
                "owned_by", "view_group", "edit_group"
            )
        ]

    context = {
        "title": title or "SQL Dashboard",
        "html_title": html_title,
        "query_results": query_results,
        "unverified_sql_queries": unverified_sql_queries,
        "available_tables": available_tables,
        "description": description,
        "dashboard": dashboard,
        "saved_dashboard": bool(dashboard),
        "user_owns_dashboard": dashboard and request.user == dashboard.owned_by,
        "user_can_edit_dashboard": dashboard and dashboard.user_can_edit(request.user),
        "user_can_execute_sql": user_can_execute_sql,
        "user_can_export_data": getattr(settings, "DASHBOARD_ENABLE_FULL_EXPORT", None)
        and user_can_execute_sql,
        "parameter_values": parameter_values.items(),
        "too_long_so_use_post": too_long_so_use_post,
        "saved_dashboards": saved_dashboards,
    }

    if extra_context:
        context.update(extra_context)

    response = render(
        request,
        template,
        context,
    )
    if request.user.is_authenticated:
        response["cache-control"] = "private"
    response["Content-Security-Policy"] = "frame-ancestors 'self'"
    return response


def dashboard(request, slug):
    dashboard = get_object_or_404(Dashboard, slug=slug)
    # Can current user see it, based on view_policy?
    view_policy = dashboard.view_policy
    owner = dashboard.owned_by
    denied = HttpResponseForbidden("You cannot access this dashboard")
    denied["cache-control"] = "private"
    if view_policy == Dashboard.ViewPolicies.PRIVATE:
        if request.user != owner:
            return denied
    elif view_policy == Dashboard.ViewPolicies.LOGGEDIN:
        if not request.user.is_authenticated:
            return denied
    elif view_policy == Dashboard.ViewPolicies.GROUP:
        if (not request.user.is_authenticated) or not (
            request.user == owner
            or request.user.groups.filter(pk=dashboard.view_group_id).exists()
        ):
            return denied
    elif view_policy == Dashboard.ViewPolicies.STAFF:
        if (not request.user.is_authenticated) or (
            request.user != owner and not request.user.is_staff
        ):
            return denied
    elif view_policy == Dashboard.ViewPolicies.SUPERUSER:
        if (not request.user.is_authenticated) or (
            request.user != owner and not request.user.is_superuser
        ):
            return denied
    return _dashboard_index(
        request,
        sql_queries=[query.sql for query in dashboard.queries.all()],
        title=dashboard.title,
        description=dashboard.description,
        dashboard=dashboard,
        template="django_sql_dashboard/saved_dashboard.html",
    )


non_alpha_re = re.compile(r"[^a-zA-Z0-9]")


def export_sql_results(request):
    export_key = [k for k in request.POST.keys() if k.startswith("export_")][0]
    _, format, sql_index = export_key.split("_")
    assert format in ("csv", "tsv")
    sqls = request.POST.getlist("sql")
    sql = sqls[int(sql_index)]
    parameter_values = {
        parameter: request.POST.get(parameter, "")
        for parameter in extract_named_parameters(sql)
    }
    alias = getattr(settings, "DASHBOARD_DB_ALIAS", "dashboard")
    # Decide on filename
    sql_hash = hashlib.sha256(sql.encode("utf-8")).hexdigest()[:6]
    filename = non_alpha_re.sub("-", sql.lower()[:30]) + sql_hash

    filename_plus_ext = filename + "." + format

    connection = connections[alias]
    connection.cursor()  # To initialize connection
    cursor = connection.create_cursor(name="c" + filename.replace("-", "_"))

    csvfile = StringIO()
    csvwriter = csv.writer(
        csvfile,
        dialect={
            "csv": csv.excel,
            "tsv": csv.excel_tab,
        }[format],
    )

    def read_and_flush():
        csvfile.seek(0)
        data = csvfile.read()
        csvfile.seek(0)
        csvfile.truncate()
        return data

    def rows():
        try:
            cursor.execute(sql, parameter_values)
            done_header = False
            while True:
                records = cursor.fetchmany(size=2000)
                if not done_header:
                    csvwriter.writerow([r.name for r in cursor.description])
                    yield read_and_flush()
                    done_header = True
                if not records:
                    break
                for record in records:
                    csvwriter.writerow(record)
                    yield read_and_flush()
        finally:
            cursor.close()

    response = StreamingHttpResponse(
        rows(),
        content_type={
            "csv": "text/csv",
            "tsv": "text/tab-separated-values",
        }[format],
    )
    response["Content-Disposition"] = 'attachment; filename="' + filename_plus_ext + '"'
    return response



# Generated by Django 3.1.7 on 2021-03-13 04:32

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Dashboard",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("slug", models.SlugField(unique=True)),
                ("title", models.CharField(blank=True, max_length=128)),
                ("description", models.TextField(blank=True)),
            ],
            options={
                "permissions": [("execute_sql", "Can execute arbitrary SQL queries")],
            },
        ),
        migrations.CreateModel(
            name="DashboardQuery",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("sql", models.TextField()),
                (
                    "dashboard",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="queries",
                        to="django_sql_dashboard.dashboard",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Dashboard queries",
                "order_with_respect_to": "dashboard",
            },
        ),
    ]



import csv
import io
import json

import bleach
import markdown
from django import template
from django.utils.html import escape, urlize
from django.utils.safestring import mark_safe

from ..utils import sign_sql as sign_sql_original

TAGS = [
    "a",
    "abbr",
    "acronym",
    "b",
    "blockquote",
    "code",
    "em",
    "i",
    "li",
    "ol",
    "strong",
    "ul",
    "pre",
    "p",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
]
ATTRIBUTES = {"a": ["href"]}

register = template.Library()


@register.filter
def sign_sql(value):
    return sign_sql_original(value)


@register.filter
def sql_dashboard_bleach(value):
    return mark_safe(
        bleach.clean(
            value,
            tags=TAGS,
            attributes=ATTRIBUTES,
        )
    )


@register.filter
def sql_dashboard_markdown(value):
    return mark_safe(
        bleach.linkify(
            bleach.clean(
                markdown.markdown(
                    value,
                    output_format="html5",
                ),
                tags=TAGS,
                attributes=ATTRIBUTES,
            )
        )
    )


@register.filter
def sql_dashboard_tsv(result):
    writer = io.StringIO()
    csv_writer = csv.writer(writer, delimiter="\t")
    csv_writer.writerow(result["columns"])
    for row in result["row_lists"]:
        csv_writer.writerow(row)
    return writer.getvalue().strip()


@register.filter
def format_cell(value):
    if isinstance(value, str) and value and value[0] in ("{", "["):
        try:
            return mark_safe(
                '<pre class="json">{}</pre>'.format(
                    escape(json.dumps(json.loads(value), indent=2))
                )
            )
        except json.JSONDecodeError:
            pass
    return mark_safe(urlize(value, nofollow=True, autoescape=True))



import binascii
import json
import re
import urllib.parse
from collections import namedtuple

from django.core import signing

SQL_SALT = "django_sql_dashboard:query"

signer = signing.Signer(salt=SQL_SALT)


def sign_sql(sql):
    return signer.sign(sql)


def unsign_sql(signed_sql, try_object=False):
    # Returns (sql, signature_verified)
    # So we can handle broken signatures
    # Usually this will be a regular string
    try:
        sql = signer.unsign(signed_sql)
        return sql, True
    except signing.BadSignature:
        try:
            value, bad_sig = signed_sql.rsplit(signer.sep, 1)
            return value, False
        except ValueError:
            return signed_sql, False


class Row:
    def __init__(self, values, columns):
        self.values = values
        self.columns = columns
        self.zipped = dict(zip(columns, values))

    def __getitem__(self, key):
        if isinstance(key, int):
            return self.values[key]
        else:
            return self.zipped[key]

    def __repr__(self):
        return json.dumps(self.zipped)


def displayable_rows(rows):
    fixed = []
    for row in rows:
        fixed_row = []
        for cell in row:
            if isinstance(cell, (dict, list)):
                cell = json.dumps(cell)
            fixed_row.append(cell)
        fixed.append(fixed_row)
    return fixed


_named_parameters_re = re.compile(r"\%\(([^\)]+)\)s")


def extract_named_parameters(sql):
    params = _named_parameters_re.findall(sql)
    # Validation step: after removing params, are there
    # any single `%` symbols that will confuse psycopg2?
    without_params = _named_parameters_re.sub("", sql)
    without_double_percents = without_params.replace("%%", "")
    if "%" in without_double_percents:
        raise ValueError(r"Found a single % character")
    return params


def check_for_base64_upgrade(queries):
    if not queries:
        return
    # Strip of the timing bit if there is one
    queries = [q.split(":")[0] for q in queries]
    # If every query is base64-encoded JSON, return a new querystring
    if not all(is_valid_base64_json(query) for query in queries):
        return
    # Need to decode these and upgrade them to ?sql= links
    sqls = []
    for query in queries:
        sqls.append(sign_sql(json.loads(signing.b64_decode(query.encode()))))
    return "?" + urllib.parse.urlencode({"sql": sqls}, True)


def is_valid_base64_json(s):
    try:
        json.loads(signing.b64_decode(s.encode()))
        return True
    except (json.JSONDecodeError, binascii.Error, UnicodeDecodeError):
        return False



from django.conf import settings
from django.db import models
from django.urls import reverse
from django.utils import timezone


class Dashboard(models.Model):
    slug = models.SlugField(unique=True)
    title = models.CharField(blank=True, max_length=128)
    description = models.TextField(blank=True)
    owned_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="owned_dashboards",
        help_text="User who owns this dashboard",
    )
    created_at = models.DateTimeField(default=timezone.now)

    class ViewPolicies(models.TextChoices):
        PRIVATE = ("private", "Private")
        PUBLIC = ("public", "Public")
        UNLISTED = ("unlisted", "Unlisted")
        LOGGEDIN = ("loggedin", "Logged-in users")
        GROUP = ("group", "Users in group")
        STAFF = ("staff", "Staff users")
        SUPERUSER = ("superuser", "Superusers")

    class EditPolicies(models.TextChoices):
        PRIVATE = ("private", "Private")
        LOGGEDIN = ("loggedin", "Logged-in users")
        GROUP = ("group", "Users in group")
        STAFF = ("staff", "Staff users")
        SUPERUSER = ("superuser", "Superusers")

    # Permissions
    view_policy = models.CharField(
        max_length=10,
        choices=ViewPolicies.choices,
        default=ViewPolicies.PRIVATE,
        help_text="Who can view this dashboard",
    )
    edit_policy = models.CharField(
        max_length=10,
        choices=EditPolicies.choices,
        default=EditPolicies.PRIVATE,
        help_text="Who can edit this dashboard",
    )
    view_group = models.ForeignKey(
        "auth.Group",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="can_view_dashboards",
        help_text="Group that can view, for 'Users in group' policy",
    )
    edit_group = models.ForeignKey(
        "auth.Group",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="can_edit_dashboards",
        help_text="Group that can edit, for 'Users in group' policy",
    )

    def __str__(self):
        return self.title or self.slug

    def view_summary(self):
        s = self.get_view_policy_display()
        if self.view_policy == "group":
            s += ' "{}"'.format(self.view_group)
        return s

    def get_absolute_url(self):
        return reverse("django_sql_dashboard-dashboard", args=[self.slug])

    def get_edit_url(self):
        return reverse("admin:django_sql_dashboard_dashboard_change", args=(self.id,))

    class Meta:
        permissions = [("execute_sql", "Can execute arbitrary SQL queries")]

    def user_can_edit(self, user):
        if not user:
            return False
        if self.owned_by == user:
            return True
        if self.edit_policy == self.EditPolicies.LOGGEDIN:
            return True
        if self.edit_policy == self.EditPolicies.STAFF and user.is_staff:
            return True
        if self.edit_policy == self.EditPolicies.SUPERUSER and user.is_superuser:
            return True
        if (
            self.edit_policy == self.EditPolicies.GROUP
            and self.edit_group
            and self.edit_group.user_set.filter(pk=user.pk).exists()
        ):
            return True
        return False

    @classmethod
    def get_visible_to_user(cls, user):
        allowed_policies = [cls.ViewPolicies.PUBLIC, cls.ViewPolicies.LOGGEDIN]
        if user.is_staff:
            allowed_policies.append(cls.ViewPolicies.STAFF)
        if user.is_superuser:
            allowed_policies.append(cls.ViewPolicies.SUPERUSER)
        return (
            cls.objects.filter(
                models.Q(owned_by=user)
                | models.Q(view_policy__in=allowed_policies)
                | models.Q(view_policy=cls.ViewPolicies.GROUP, view_group__user=user)
            )
            .annotate(
                is_owner=models.ExpressionWrapper(
                    models.Q(owned_by__exact=user.pk),
                    output_field=models.BooleanField(),
                )
            )
            .order_by("-is_owner", "slug")
        ).distinct()


class DashboardQuery(models.Model):
    dashboard = models.ForeignKey(
        Dashboard, related_name="queries", on_delete=models.CASCADE
    )
    sql = models.TextField()

    def __str__(self):
        return self.sql

    class Meta:
        verbose_name_plural = "Dashboard queries"
        order_with_respect_to = "dashboard"



# Generated by Django 3.1.7 on 2021-05-08 15:44

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("django_sql_dashboard", "0002_dashboard_permissions"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dashboard",
            name="edit_group",
            field=models.ForeignKey(
                blank=True,
                help_text="Group that can edit, for 'Users in group' policy",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="can_edit_dashboards",
                to="auth.group",
            ),
        ),
        migrations.AlterField(
            model_name="dashboard",
            name="edit_policy",
            field=models.CharField(
                choices=[
                    ("private", "Private"),
                    ("loggedin", "Logged-in users"),
                    ("group", "Users in group"),
                    ("staff", "Staff users"),
                    ("superuser", "Superusers"),
                ],
                default="private",
                help_text="Who can edit this dashboard",
                max_length=10,
            ),
        ),
        migrations.AlterField(
            model_name="dashboard",
            name="owned_by",
            field=models.ForeignKey(
                blank=True,
                help_text="User who owns this dashboard",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="owned_dashboards",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="dashboard",
            name="view_group",
            field=models.ForeignKey(
                blank=True,
                help_text="Group that can view, for 'Users in group' policy",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="can_view_dashboards",
                to="auth.group",
            ),
        ),
        migrations.AlterField(
            model_name="dashboard",
            name="view_policy",
            field=models.CharField(
                choices=[
                    ("private", "Private"),
                    ("public", "Public"),
                    ("unlisted", "Unlisted"),
                    ("loggedin", "Logged-in users"),
                    ("group", "Users in group"),
                    ("staff", "Staff users"),
                    ("superuser", "Superusers"),
                ],
                default="private",
                help_text="Who can view this dashboard",
                max_length=10,
            ),
        ),
    ]



# Generated by Django 3.1.7 on 2021-03-16 16:11

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("django_sql_dashboard", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="dashboard",
            name="created_at",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="dashboard",
            name="edit_group",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="can_edit_dashboards",
                to="auth.group",
            ),
        ),
        migrations.AddField(
            model_name="dashboard",
            name="edit_policy",
            field=models.CharField(
                choices=[
                    ("private", "Private"),
                    ("loggedin", "Logged-in users"),
                    ("group", "Users in group"),
                    ("staff", "Staff users"),
                    ("superuser", "Superusers"),
                ],
                default="private",
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="dashboard",
            name="owned_by",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="owned_dashboards",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddField(
            model_name="dashboard",
            name="view_group",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="can_view_dashboards",
                to="auth.group",
            ),
        ),
        migrations.AddField(
            model_name="dashboard",
            name="view_policy",
            field=models.CharField(
                choices=[
                    ("private", "Private"),
                    ("public", "Public"),
                    ("unlisted", "Unlisted"),
                    ("loggedin", "Logged-in users"),
                    ("group", "Users in group"),
                    ("staff", "Staff users"),
                    ("superuser", "Superusers"),
                ],
                default="private",
                max_length=10,
            ),
        ),
    ]



import os

from setuptools import setup

VERSION = "0.14"


def get_long_description():
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()


setup(
    name="django-sql-dashboard",
    description="Django app for building dashboards using raw SQL queries",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Simon Willison",
    url="https://github.com/simonw/django-sql-dashboard",
    project_urls={
        "Documentation": "https://django-sql-dashboard.datasette.io/",
        "Issues": "https://github.com/simonw/django-sql-dashboard/issues",
        "CI": "https://github.com/simonw/django-sql-dashboard/actions",
        "Changelog": "https://github.com/simonw/django-sql-dashboard/releases",
    },
    license="Apache License, Version 2.0",
    version=VERSION,
    packages=["django_sql_dashboard"],
    package_data={
        "django_sql_dashboard": [
            "templates/django_sql_dashboard/*.html",
            "templates/django_sql_dashboard/widgets/*.html",
            "migrations/*.py",
            "templatetags/*.py",
        ]
    },
    install_requires=["Django>=3.0", "markdown", "bleach"],
    extras_require={
        "test": [
            "psycopg2",
            "pytest",
            "pytest-django==4.2.0",
            "pytest-pythonpath",
            "dj-database-url",
            "testing.postgresql",
            "beautifulsoup4",
            "html5lib",
        ],
    },
    tests_require=["django-sql-dashboard[test]"],
    python_requires=">=3.6",
)
